import { RouterController, RouterHandlerType } from '../types/TRouter';
import RouterState from './RouterState';
export declare enum EVENTS {
    ROUTE_PROCESSED = "ROUTE_PROCESSED",
    ALL_ROUTES_PROCESSED = "ALL_ROUTES_PROCESSED"
}
export type BaseRouterUseType<T> = RouterController<T>[] | RouterController<T>;
export type ControllerLookupResponseType<T> = Promise<RouterHandlerType<T | void>>;
export type EventListenerCallbackType<T> = (route?: RouterController<T>) => T | void;
export type EventListenerType<T> = (event: EVENTS, route?: RouterController<T>) => void;
export type HandlerByEventMapType<T> = {
    [event in EVENTS]?: EventListenerCallbackType<T>;
};
export default abstract class BaseRouter<T> extends RouterState<T> {
    static separator: string;
    protected __allRoutesLoaded: boolean;
    private __isExposed;
    private __totalOfRoutes;
    private __routesProccesed;
    private __eventListener;
    constructor();
    /**
     * defined method to lookup for a resource controller. MUST be implemented
     * @param receivedMethod
     * @param receivedPath
     */
    protected lookup(receivedMethod: string, receivedPath: string): ControllerLookupResponseType<T>;
    protected addEventListener(listenerHandler: EventListenerType<T>): void;
    private emit;
    private emitPartial;
    protected lookupInCustomRouter(eventMapLookupController: HandlerByEventMapType<T>): Promise<T | void>;
    protected getMapKey(route: Pick<RouterController<T>, 'path' | 'method'>, separator?: string): string;
    private useController;
    useAll(routes: BaseRouterUseType<T>, overrides?: Partial<RouterController<T>>): void;
    /**
     * Overloaded method.
     *
     * If args is type of [string,RouterController<T>] add a single controller for that path. Usually you call it when using the util createController.
     * if args is type of [RouterController<T>[]] add a bulk of paths.
     * @param args All arguments.
     */
    use(...args: [string, BaseRouterUseType<T>] | [BaseRouterUseType<T> | any] | [BaseRouterUseType<T>[] | any]): void;
    delete(path: string): T;
    expose(): void;
    static errorToHttpError(e: any): {
        statusCode: any;
        body: any;
    };
}
