import { IDataStructureRequiredIndexType } from '../types/DataStructures';
import { IDataStructure } from '../interfaces/IDataStructure';
export declare const RouterRootBase = "/";
export declare enum METHODS {
    connect = "connect",
    delete = "delete",
    get = "get",
    head = "head",
    options = "options",
    post = "post",
    put = "put",
    patch = "patch",
    trace = "trace"
}
export declare const MethodSymbols: {
    connect: symbol;
    delete: symbol;
    get: symbol;
    head: symbol;
    options: symbol;
    post: symbol;
    put: symbol;
    patch: symbol;
    trace: symbol;
};
export type TreeNodeMethods<T> = {
    [method in METHODS]?: T;
};
export type ControllerMethods<T> = TreeNodeMethods<T>;
export type TreeRoot<T> = {
    [path: string]: TreeRoot<T>;
} & TreeNodeMethods<T>;
export declare enum IteratorResultTypes {
    END = "END",
    NOT_FOUND = "NOT_FOUND"
}
export type IteratorResultType<T> = {
    status: IteratorResultTypes;
    tree?: TreeRoot<T>;
    tokens?: string[];
    currentToken?: string;
};
export type MatchSingleResultType<T> = {
    path: string;
    tree?: TreeRoot<T>;
};
export type MatchMultipleResults<T> = MatchSingleResultType<T>[];
export type MatchResult<T> = MatchMultipleResults<T> | MatchSingleResultType<T> | undefined;
export type RouterRoot<T> = {
    [RouterRootBase]: TreeRoot<T>;
};
export default class RouterState<T> implements IDataStructure<T> {
    static SEPARATOR: string;
    private state;
    constructor(initialState?: RouterRoot<T>);
    getRoot(): TreeRoot<T>;
    getSymbolMethod(method: string): any;
    matchExact(token: string, tree: TreeRoot<T>): MatchSingleResultType<T>;
    /**
     * It wil try to match with a regular expresion.
     * Before using it first try to matchExact function
     * @param token
     * @param tree
     * @returns
     */
    matchWith(token: string, tree: TreeRoot<T>): MatchMultipleResults<T>;
    match(token: string, tree: TreeRoot<T>): MatchMultipleResults<T>;
    iterate(tokens: string[], tree: TreeRoot<T>): IteratorResultType<T>;
    search(tokens: string[], tree: TreeRoot<T>): TreeRoot<T> | undefined;
    /**
     * Given an index return the value of that node
     * @param index IDataStructureIndexType
     * @returns T
     */
    get(index: string): T | undefined;
    /**
     * Insert an element T at the given index
     * @param index IDataStructureIndexType index where the value will be stored
     * @param value
     */
    insert(index: string, value: T): T;
    remove(index: IDataStructureRequiredIndexType): T;
    update(index: IDataStructureRequiredIndexType): {
        new: T;
        old: T;
    };
}
